@@ -1,5 +1,6 @@
 """Support for Amazon devices."""
 
+import asyncio
 import base64
 import hashlib
 import mimetypes
@@ -9,18 +10,19 @@ from dataclasses import dataclass
 from datetime import UTC, datetime, timedelta
 from enum import StrEnum
 from http import HTTPMethod, HTTPStatus
-from http.cookies import Morsel, SimpleCookie
+from http.cookies import Morsel
 from pathlib import Path
 from typing import Any, cast
 from urllib.parse import parse_qs, urlencode
 
 import orjson
-from aiohttp import ClientConnectorError, ClientResponse, ClientSession
+from aiohttp import ClientConnectorError, ClientResponse, ClientSession, CookieJar
 from bs4 import BeautifulSoup, Tag
 from langcodes import Language
-from multidict import CIMultiDictProxy, MultiDictProxy
+from multidict import MultiDictProxy
 from yarl import URL
 
+from . import __version__
 from .const import (
     _LOGGER,
     AMAZON_APP_BUNDLE_ID,
@@ -84,6 +86,7 @@ class AmazonDevice:
     device_type: str
     device_owner_customer_id: str
     device_cluster_members: list[str]
+    device_locale: str
     online: bool
     serial_number: str
     software_version: str
@@ -103,6 +106,7 @@ class AmazonSequenceType(StrEnum):
     Sound = "Alexa.Sound"
     Music = "Alexa.Music.PlaySearchPhrase"
     TextCommand = "Alexa.TextCommand"
+    LaunchSkill = "Alexa.Operation.SkillConnections.Launch"
 
 
 class AmazonMusicSource(StrEnum):
@@ -127,19 +131,19 @@ class AmazonEchoApi:
         # Force country digits as lower case
         country_code = login_country_code.lower()
 
-        locale = DOMAIN_BY_ISO3166_COUNTRY.get(country_code)
-        domain = locale["domain"] if locale else country_code
+        locale = DOMAIN_BY_ISO3166_COUNTRY.get(country_code, {})
+        domain = locale.get("domain", country_code)
+        market = locale.get("market", f"https://www.amazon.{domain}")
+        assoc_handle = locale.get(
+            "openid.assoc_handle", f"{DEFAULT_ASSOC_HANDLE}_{country_code}"
+        )
 
-        if locale and (assoc := locale.get("openid.assoc_handle")):
-            assoc_handle = assoc
-        else:
-            assoc_handle = f"{DEFAULT_ASSOC_HANDLE}_{country_code}"
         self._assoc_handle = assoc_handle
-
         self._login_email = login_email
         self._login_password = login_password
         self._login_country_code = country_code
         self._domain = domain
+        self._market = market
         self._cookies = self._build_init_cookies()
         self._csrf_cookie: str | None = None
         self._save_raw_data = save_raw_data
@@ -149,20 +153,18 @@ class AmazonEchoApi:
 
         self.session: ClientSession
         self._devices: dict[str, Any] = {}
+        self._sensors_available: bool = True
 
-        if locale and (lang := locale.get("language")):
-            language = lang
-        else:
-            lang_object = Language.make(territory=self._login_country_code.upper())
-            lang_maximized = lang_object.maximize()
-            language = f"{lang_maximized.language}-{lang_maximized.region}"
-
-        self._language = language
+        lang_object = Language.make(territory=self._login_country_code.upper())
+        lang_maximized = lang_object.maximize()
+        self._language = f"{lang_maximized.language}-{lang_maximized.region}"
 
         _LOGGER.debug(
-            "Initialize library with domain <%s> and language <%s>",
+            "Initialize library v%s: domain <amazon.%s>, language <%s>, market: <%s>",
+            __version__,
             self._domain,
             self._language,
+            self._market,
         )
 
     def _load_website_cookies(self) -> dict[str, str]:
@@ -303,29 +305,9 @@ class AmazonEchoApi:
         """Create HTTP client session."""
         if not hasattr(self, "session") or self.session.closed:
             _LOGGER.debug("Creating HTTP session (aiohttp)")
-            headers = DEFAULT_HEADERS
-            headers.update({"Accept-Language": self._language})
-            self.session = ClientSession(
-                headers=headers,
-                cookies=self._cookies,
-            )
-
-    async def _parse_cookies_from_headers(
-        self, headers: CIMultiDictProxy[str]
-    ) -> dict[str, str]:
-        """Parse cookies with a value from headers."""
-        cookies_with_value: dict[str, str] = {}
-
-        for value in headers.getall("Set-Cookie", ()):
-            cookie = SimpleCookie()
-            cookie.load(value)
-
-            for name, morsel in cookie.items():
-                if morsel.value and morsel.value not in ("-", ""):
-                    cookies_with_value[name] = morsel.value
-
-        _LOGGER.debug("Cookies from headers: %s", cookies_with_value)
-        return cookies_with_value
+            cookie_jar = CookieJar()
+            cookie_jar.update_cookies(self._cookies)
+            self.session = ClientSession(cookie_jar=cookie_jar)
 
     async def _ignore_ap_signin_error(self, response: ClientResponse) -> bool:
         """Return true if error is due to signin endpoint."""
@@ -338,6 +320,14 @@ class AmazonEchoApi:
             )
         return False
 
+    async def _ignore_phoenix_error(self, response: ClientResponse) -> bool:
+        """Return true if error is due to phoenix endpoint."""
+        # Endpoint URI_IDS replies with error 199 or 299
+        # during maintenance
+        return response.status in [HTTP_ERROR_199, HTTP_ERROR_299] and (
+            URI_IDS in response.url.path
+        )
+
     async def _http_phrase_error(self, error: int) -> str:
         """Convert numeric error in human phrase."""
         if error == HTTP_ERROR_199:
@@ -372,29 +362,52 @@ class AmazonEchoApi:
             headers.update({"User-Agent": DEFAULT_AGENT})
         if self._csrf_cookie:
             csrf = {CSRF_COOKIE: self._csrf_cookie}
-            _LOGGER.debug("Adding <%s> to headers", csrf)
+            _LOGGER.debug("Adding to headers: %s", csrf)
             headers.update(csrf)
 
         if json_data:
             json_header = {"Content-Type": "application/json; charset=utf-8"}
-            _LOGGER.debug("Adding %s to headers", json_header)
+            _LOGGER.debug("Adding to headers: %s", json_header)
             headers.update(json_header)
 
         _cookies = (
             self._load_website_cookies() if self._login_stored_data else self._cookies
         )
-        try:
-            resp = await self.session.request(
-                method,
-                URL(url, encoded=True),
-                data=input_data if not json_data else orjson.dumps(input_data),
-                cookies=_cookies,
-                headers=headers,
-            )
-        except (TimeoutError, ClientConnectorError) as exc:
-            raise CannotConnect(f"Connection error during {method}") from exc
+        self.session.cookie_jar.update_cookies(_cookies)
+
+        resp: ClientResponse | None = None
+        for delay in [0, 1, 2, 5, 8, 12, 21]:
+            if delay:
+                _LOGGER.info(
+                    "Sleeping for %s seconds before retrying API call to %s", delay, url
+                )
+                await asyncio.sleep(delay)
+
+            try:
+                resp = await self.session.request(
+                    method,
+                    URL(url, encoded=True),
+                    data=input_data if not json_data else orjson.dumps(input_data),
+                    headers=headers,
+                )
+
+            except (TimeoutError, ClientConnectorError) as exc:
+                _LOGGER.warning("Connection error to %s: %s", url, repr(exc))
+                raise CannotConnect(f"Connection error during {method}") from exc
+
+            # Retry with a delay only for specific HTTP status
+            # that can benefits of a back-off
+            if resp.status not in [
+                HTTPStatus.INTERNAL_SERVER_ERROR,
+                HTTPStatus.SERVICE_UNAVAILABLE,
+                HTTPStatus.TOO_MANY_REQUESTS,
+            ]:
+                break
+
+        if resp is None:
+            _LOGGER.error("No response received from %s", url)
+            raise CannotConnect(f"No response received from {url}")
 
-        self._cookies.update(**await self._parse_cookies_from_headers(resp.headers))
         if not self._csrf_cookie:
             self._csrf_cookie = resp.cookies.get(CSRF_COOKIE, Morsel()).value
             _LOGGER.debug("CSRF cookie value: <%s>", self._csrf_cookie)
@@ -413,10 +426,12 @@ class AmazonEchoApi:
                 HTTPStatus.PROXY_AUTHENTICATION_REQUIRED,
                 HTTPStatus.UNAUTHORIZED,
             ]:
-                raise CannotAuthenticate(self._http_phrase_error(resp.status))
-            if not await self._ignore_ap_signin_error(resp):
+                raise CannotAuthenticate(await self._http_phrase_error(resp.status))
+            if not await self._ignore_ap_signin_error(
+                resp
+            ) and not await self._ignore_phoenix_error(resp):
                 raise CannotRetrieveData(
-                    f"Request failed: {self._http_phrase_error(resp.status)}"
+                    f"Request failed: {await self._http_phrase_error(resp.status)}"
                 )
 
         await self._save_to_file(
@@ -527,7 +542,9 @@ class AmazonEchoApi:
                 obfuscate_email(self._login_email),
                 msg,
             )
-            raise CannotRegisterDevice(f"{self._http_phrase_error(resp.status)}: {msg}")
+            raise CannotRegisterDevice(
+                f"{await self._http_phrase_error(resp.status)}: {msg}"
+            )
 
         success_response = resp_json["response"]["success"]
 
@@ -571,21 +588,18 @@ class AmazonEchoApi:
             raise CannotAuthenticate
 
         resp_me_json = await resp_me.json()
-        market = resp_me_json["marketPlaceDomainName"]
-        language = resp_me_json["marketPlaceLocale"]
+        amazon_market = resp_me_json["marketPlaceDomainName"]
 
-        _domain = f"https://www.amazon.{self._domain}"
-
-        if market != _domain or language != self._language:
-            _LOGGER.debug(
-                "Selected country <%s> doesn't matches Amazon account:\n%s\n vs \n%s",
+        if amazon_market != self._market:
+            _LOGGER.warning(
+                "Selected country <%s> doesn't matches Amazon API reply:\n%s\n vs \n%s",
                 self._login_country_code.upper(),
-                {"site  ": _domain, "locale": self._language},
-                {"market": market, "locale": language},
+                {"input ": self._market},
+                {"amazon": amazon_market},
             )
             raise WrongCountry
 
-        _LOGGER.debug("User selected country matches Amazon account one")
+        _LOGGER.debug("User selected country matches Amazon API one")
 
     async def _get_devices_ids(self) -> list[dict[str, str]]:
         """Retrieve devices entityId and applianceId."""
@@ -594,6 +608,17 @@ class AmazonEchoApi:
             url=f"https://alexa.amazon.{self._domain}{URI_IDS}",
             amazon_user_agent=False,
         )
+
+        # Sensors data not available
+        if raw_resp.status != HTTPStatus.OK:
+            _LOGGER.warning(
+                "Sensors data not available [%s error '%s'], skipping",
+                URI_IDS,
+                await self._http_phrase_error(raw_resp.status),
+            )
+            self._sensors_available = False
+            return []
+
         json_data = await raw_resp.json()
 
         network_detail = orjson.loads(json_data["networkDetail"])
@@ -818,10 +843,12 @@ class AmazonEchoApi:
                 else:
                     self._devices[dev_serial] = {key: data}
 
-        entity_ids_list = await self._get_devices_ids()
-        devices_sensors = (
-            await self._get_sensors_states(entity_ids_list) if entity_ids_list else {}
-        )
+        devices_sensors: dict[str, dict[str, AmazonDeviceSensor]] = {}
+
+        if self._sensors_available and (
+            entity_ids_list := await self._get_devices_ids()
+        ):
+            devices_sensors = await self._get_sensors_states(entity_ids_list)
 
         final_devices_list: dict[str, AmazonDevice] = {}
         for device in self._devices.values():
@@ -830,7 +857,7 @@ class AmazonEchoApi:
             if not devices_node or (devices_node.get("deviceType") in DEVICE_TO_IGNORE):
                 continue
 
-            preferences_node = device.get(NODE_PREFERENCES)
+            preferences_node = device.get(NODE_PREFERENCES, {})
             do_not_disturb_node = device[NODE_DO_NOT_DISTURB]
             bluetooth_node = device[NODE_BLUETOOTH]
             identifier_node = device.get(NODE_IDENTIFIER, {})
@@ -852,13 +879,12 @@ class AmazonEchoApi:
                 device_cluster_members=(
                     devices_node["clusterMembers"] or [serial_number]
                 ),
+                device_locale=preferences_node.get("locale", self._language),
                 online=devices_node["online"],
                 serial_number=serial_number,
                 software_version=devices_node["softwareVersion"],
                 do_not_disturb=do_not_disturb_node["enabled"],
-                response_style=(
-                    preferences_node["responseStyle"] if preferences_node else None
-                ),
+                response_style=preferences_node.get("responseStyle"),
                 bluetooth_state=bluetooth_node["online"],
                 entity_id=identifier_node.get("entityId"),
                 appliance_id=identifier_node.get("applianceId"),
@@ -903,7 +929,7 @@ class AmazonEchoApi:
         )
         if not model_details:
             _LOGGER.warning(
-                "Unknown device type '%s' for %s: please read https://github.com/chemelli74/aioamazondevices?tab=readme-ov-file#unknown-device-type",
+                "Unknown device type '%s' for %s: please read https://github.com/chemelli74/aioamazondevices/wiki/Unknown-Device-Types",
                 device.device_type,
                 device.account_name,
             )
@@ -925,7 +951,7 @@ class AmazonEchoApi:
         base_payload = {
             "deviceType": device.device_type,
             "deviceSerialNumber": device.serial_number,
-            "locale": self._language,
+            "locale": device.device_locale,
             "customerId": device.device_owner_customer_id,
         }
 
@@ -960,7 +986,7 @@ class AmazonEchoApi:
                 "expireAfter": "PT5S",
                 "content": [
                     {
-                        "locale": self._language,
+                        "locale": device.device_locale,
                         "display": {
                             "title": "Home Assistant",
                             "body": message_body,
@@ -996,6 +1022,19 @@ class AmazonEchoApi:
                 "skillId": "amzn1.ask.1p.tellalexa",
                 "text": message_body,
             }
+        elif message_type == AmazonSequenceType.LaunchSkill:
+            payload = {
+                **base_payload,
+                "targetDevice": {
+                    "deviceType": device.device_type,
+                    "deviceSerialNumber": device.serial_number,
+                },
+                "connectionRequest": {
+                    "uri": "connection://AMAZON.Launch/" + message_body,
+                },
+            }
+        else:
+            raise ValueError(f"Message type <{message_type}> is not recognised")
 
         sequence = {
             "@type": "com.amazon.alexa.behaviors.model.Sequence",
@@ -1074,6 +1113,16 @@ class AmazonEchoApi:
             device, AmazonSequenceType.TextCommand, message_body
         )
 
+    async def call_alexa_skill(
+        self,
+        device: AmazonDevice,
+        message_body: str,
+    ) -> None:
+        """Call Alexa.LaunchSkill to launch a skill."""
+        return await self._send_message(
+            device, AmazonSequenceType.LaunchSkill, message_body
+        )
+
     async def set_do_not_disturb(self, device: AmazonDevice, state: bool) -> None:
         """Set do_not_disturb flag."""
         payload = {
